/*   msXpertSuite - mass spectrometry software suite
 *   -----------------------------------------------
 *   Copyright(C) 2016 Filippo Rusconi
 *
 *   http://www.{site}.org
 *
 *   This file is part of the msXpertSuite project.
 *
 *   The msXpertSuite project is the successor of the massXpert project. This
 *   project now includes various independent modules:
 *   - massXpert, the original mass spectrometry environment software (greatly
 *     refactored code);
 *   - mineXpert, a powerful TIC chromatogram/mass/drift spectrum viewer/miner;
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef ANALYSIS_PREFERENCES_HPP
#define ANALYSIS_PREFERENCES_HPP


/////////////////////// Qt includes
#include <QString>
#include <QHash>


/////////////////////// Local includes


namespace msXpertSuite
{

  enum RecordTarget
  {
    // the left hand value of the << : the bit value (either 0 or 1)
    // the right hand value of the << : the bit index (from right to left)
    // at which the bit value needs to be shifted to.
    RECORD_NOTHING      = 0 << 0,
    RECORD_TO_CONSOLE   = 1 << 1,
    RECORD_TO_FILE      = 1 << 2,
    RECORD_TO_CLIPBOARD = 1 << 3,
  };

  enum FormatType
  {
    TIC_CHROM = 0,
    MASS_SPEC,
    DRIFT_SPEC,
    LAST,
  };

  struct DataFormatStringSpecif
  {
    int     m_formatType;
    QString m_label;
    QString m_format;
  };


  class AnalysisPreferences
  {
    public:
    AnalysisPreferences(const AnalysisPreferences &other);
    AnalysisPreferences();
    ~AnalysisPreferences();

    QString m_fileName;

    // The key is the format type enum value that describes the type of format
    // specif that is stored in the value. The format label is typically used
    // to setup the GUI in AnalysisPreferencesDlg (the tab widget tabs' label,
    // typically).  The key is the actual format string  specif.  She defines
    // one format string per key.
    QHash<int, DataFormatStringSpecif *> m_dataFormatStringSpecifHash;

    int m_recordTarget = RecordTarget::RECORD_NOTHING;

    int m_fileOpenMode;

    AnalysisPreferences &operator=(const AnalysisPreferences &other);

    QString asText() const;
  };

} // namespace msXpertSuite


#endif // ANALYSIS_PREFERENCES_HPP
