/*   msXpertSuite - mass spectrometry software suite
 *   -----------------------------------------------
 *   Copyright(C) 2016 Filippo Rusconi
 *
 *   http://www.{site}.org
 *
 *   This file is part of the msXpertSuite project.
 *
 *   The msXpertSuite project is the successor of the massXpert project. This
 *   project now includes various independent modules:
 *   - massXpert, the original mass spectrometry environment software (greatly
 *     refactored code);
 *   - mobXpert, a module to process and mine mobility mass spectrometry data;
 *   - viewXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


/////////////////////// Qt includes
#include <QDir>
#include <QDebug>
#include <QSqlDatabase>
#include <QRegExp>


/////////////////////// Local includes
#include "Base64EncDec.hpp"

namespace msXpertSuite
{

  Base64EncDec::Base64EncDec()
  {
  }


  Base64EncDec::~Base64EncDec()
  {
  }


  bool
  Base64EncDec::decode(const QByteArray *srcArray, QByteArray **destArray,
                       int compressionType)
  {
    if(srcArray == nullptr || destArray == nullptr)
      qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);

    if(srcArray->size() < 4)
      return false;

    // Whatever happens, we need to decode, so let's start with this.

    QByteArray *p_decByteArray =
      new QByteArray(QByteArray::fromBase64(*srcArray));

    if(compressionType == DATA_COMPRESSION_NONE)
      {
        *destArray = p_decByteArray;

        return true;
      }

    if(compressionType == DATA_COMPRESSION_ZLIB)
      {
        // Let's craft a brand new QByteArray to front-store the data in a
        // format that is fine for qUncompress() which needs some information
        // on the size of the data.
        QByteArray forZlib;
        forZlib.resize(4);
        forZlib[0] = (p_decByteArray->size() & 0xff000000) >> 24;
        forZlib[1] = (p_decByteArray->size() & 0x00ff0000) >> 16;
        forZlib[2] = (p_decByteArray->size() & 0x0000ff00) >> 8;
        forZlib[3] = (p_decByteArray->size() & 0x000000ff);
        forZlib += *p_decByteArray;

        // Store the address of the newly allocated byte array in the member
        // **QByteArray.
        *destArray = new QByteArray(qUncompress(forZlib));

        delete p_decByteArray;

        return true;
      }

    // We should not reach here.
    delete p_decByteArray;
    return false;
  }


} // namespace msXpertSuite
