/////////////////////// Qt includes
#include <QDebug>

/////////////////////// Local includes
#include "History.hpp"


namespace msXpertSuite
{

	HistoryItem::HistoryItem()
	{
		m_dateTime = QDateTime::currentDateTimeUtc();

		initializeIntegrationTypes();
	}


	HistoryItem::HistoryItem(const HistoryItem &other)
	{

		// Fill in all the Integration types that we know in the "static" QMap
		// member.

		initializeIntegrationTypes();

		m_dateTime = other.m_dateTime;

		// And now duplicate the map.
		QList<IntegrationRange *> list = other.m_integrationMap.values();

		for(int iter = 0; iter < list.size(); ++iter)
		{
			IntegrationRange *intRange = list.at(iter);
			IntegrationType intType = other.m_integrationMap.key(intRange);

			IntegrationRange *newIntRange = new IntegrationRange;
			newIntRange->start = intRange->start;
			newIntRange->end = intRange->end;

			m_integrationMap.insert(intType, newIntRange);
		}
	}


	HistoryItem &
		HistoryItem::operator=(const HistoryItem &other)
		{
			// Fill in all the Integration types that we know in the "static" QMap
			// member.

			initializeIntegrationTypes();

			m_dateTime = other.m_dateTime;

			// And now duplicate the map.
			QList<IntegrationRange *> list = other.m_integrationMap.values();

			for(int iter = 0; iter < list.size(); ++iter)
			{
				IntegrationRange *intRange = list.at(iter);
				IntegrationType intType = other.m_integrationMap.key(intRange);

				IntegrationRange *newIntRange = new IntegrationRange;
				newIntRange->start = intRange->start;
				newIntRange->end = intRange->end;

				m_integrationMap.insert(intType, newIntRange);
			}

			return *this;
		}


	HistoryItem::~HistoryItem()
	{
		QList<IntegrationRange *> irList = m_integrationMap.values();

		while(irList.size())
			delete irList.takeFirst();

		// Finally, clear the map.
		m_integrationMap.clear();

		QList<IntegrationTypeDesc *> itdList = m_intTypeDescMap.values();

		while(itdList.size())
			delete itdList.takeFirst();

		// Finally, clear the map.
		m_intTypeDescMap.clear();
	}


	void
		HistoryItem::initializeIntegrationTypes()
		{

			// The m_intTypeDescMap member is "static". Fill-in the data once and for
			// all.

			IntegrationType intType = IntegrationType::NOT_SET;
			IntegrationTypeDesc *intTypeDesc = new IntegrationTypeDesc;
			intTypeDesc->brief = "NOT_SET";
			intTypeDesc->detailed = "Not set";
			m_intTypeDescMap.insert(intType, intTypeDesc);

			intType = IntegrationType::DATA_TO_RT;
			intTypeDesc = new IntegrationTypeDesc;
			intTypeDesc->brief = "[FILE->RT]";
			intTypeDesc->detailed = "Tic from data";
			m_intTypeDescMap.insert(intType, intTypeDesc);

			intType = IntegrationType::DATA_TO_DT_MZ_DT_RANGE;
			intTypeDesc = new IntegrationTypeDesc;
			intTypeDesc->brief = "[FILE->DT_MZ (DT range)]";
			intTypeDesc->detailed =
				"Mass spectrum vs drift spectrum from data (DT range)";
			m_intTypeDescMap.insert(intType, intTypeDesc);

			intType = IntegrationType::DATA_TO_DT_MZ_MZ_RANGE;
			intTypeDesc = new IntegrationTypeDesc;
			intTypeDesc->brief = "[FILE->DT_MZ (MZ range)]";
			intTypeDesc->detailed =
				"Mass spectrum vs drift spectrum from data (MZ range)";
			m_intTypeDescMap.insert(intType, intTypeDesc);

			intType = IntegrationType::DATA_TO_DT_MZ_TO_TIC_INT_DT_RANGE;
			intTypeDesc = new IntegrationTypeDesc;
			intTypeDesc->brief = "[FILE->DT_MZ->TicI (DT range)]";
			intTypeDesc->detailed =
				"TIC intensity (single value) from [mass spectrum vs drift spectrum] from data (DT range)";
			m_intTypeDescMap.insert(intType, intTypeDesc);

			intType = IntegrationType::DATA_TO_DT_MZ_TO_TIC_INT_MZ_RANGE;
			intTypeDesc = new IntegrationTypeDesc;
			intTypeDesc->brief = "[FILE->DT_MZ->TicI (MZ range)]";
			intTypeDesc->detailed =
				"TIC intensity (single value) from [mass spectrum vs drift spectrum] from data (MZ range)";
			m_intTypeDescMap.insert(intType, intTypeDesc);

			intType = IntegrationType::ANY_TO_RT;
			intTypeDesc = new IntegrationTypeDesc;
			intTypeDesc->brief = "[any-to-RT]";
			intTypeDesc->detailed = "Any to Tic";
			m_intTypeDescMap.insert(intType, intTypeDesc);

			intType = IntegrationType::RT_TO_MZ;
			intTypeDesc = new IntegrationTypeDesc;
			intTypeDesc->brief = "[RT->MZ]";
			intTypeDesc->detailed = "Mass spectrum from Tic chromatogram";
			m_intTypeDescMap.insert(intType, intTypeDesc);

			intType = IntegrationType::RT_TO_DT;
			intTypeDesc = new IntegrationTypeDesc;
			intTypeDesc->brief = "[RT->DT]";
			intTypeDesc->detailed = "Drift time spectrum from Tic chromatogram";
			m_intTypeDescMap.insert(intType, intTypeDesc);

			intType = IntegrationType::RT_TO_TIC_INT;
			intTypeDesc = new IntegrationTypeDesc;
			intTypeDesc->brief = "[RT->TicI]";
			intTypeDesc->detailed = "TIC intensity (single value) from Tic chromatogram";
			m_intTypeDescMap.insert(intType, intTypeDesc);

			intType = IntegrationType::ANY_TO_MZ;
			intTypeDesc = new IntegrationTypeDesc;
			intTypeDesc->brief = "[any-to-MZ]";
			intTypeDesc->detailed = "Any to Mass spectrum";
			m_intTypeDescMap.insert(intType, intTypeDesc);

			intType = IntegrationType::MZ_TO_RT;
			intTypeDesc = new IntegrationTypeDesc;
			intTypeDesc->brief = "[MZ->RT]";
			intTypeDesc->detailed = "Tic chromatogram from mass spectrum";
			m_intTypeDescMap.insert(intType, intTypeDesc);

			intType = IntegrationType::MZ_TO_DT;
			intTypeDesc = new IntegrationTypeDesc;
			intTypeDesc->brief = "[MZ->DT]";
			intTypeDesc->detailed = "Drift time spectrum from mass spectrum";
			m_intTypeDescMap.insert(intType, intTypeDesc);

			intType = IntegrationType::MZ_TO_TIC_INT;
			intTypeDesc = new IntegrationTypeDesc;
			intTypeDesc->brief = "[MZ->TicI]";
			intTypeDesc->detailed = "TIC intensity (single value) from mass spectrum";
			m_intTypeDescMap.insert(intType, intTypeDesc);

			intType = IntegrationType::ANY_TO_DT;
			intTypeDesc = new IntegrationTypeDesc;
			intTypeDesc->brief = "[any-to-DT]";
			intTypeDesc->detailed = "Any to Drift spectrum";
			m_intTypeDescMap.insert(intType, intTypeDesc);

			intType = IntegrationType::DT_TO_MZ;
			intTypeDesc = new IntegrationTypeDesc;
			intTypeDesc->brief = "[DT->MZ]";
			intTypeDesc->detailed = "Mass spectrum from drift time spectrum";
			m_intTypeDescMap.insert(intType, intTypeDesc);

			intType = IntegrationType::DT_TO_RT;
			intTypeDesc = new IntegrationTypeDesc;
			intTypeDesc->brief = "[DT->RT]";
			intTypeDesc->detailed = "Tic chromatogram from drift time spectrum";
			m_intTypeDescMap.insert(intType, intTypeDesc);

			intType = IntegrationType::DT_TO_TIC_INT;
			intTypeDesc = new IntegrationTypeDesc;
			intTypeDesc->brief = "[DT->TicI]";
			intTypeDesc->detailed = "TIC intensity (single value) from drift spectrum";
			m_intTypeDescMap.insert(intType, intTypeDesc);
		}


	bool
		HistoryItem::operator==(const HistoryItem &other) const
		{
			if(this == &other)
				return true;

			if(m_dateTime != other.m_dateTime)
				return false;

			// Now we need to compare all the pairs in the map.

			if(m_integrationMap.size() != other.m_integrationMap.size())
				return false;

			QList<IntegrationType> intTypeList = m_integrationMap.keys();

			for(int iter = 0; iter < intTypeList.size(); ++iter)
			{
				IntegrationType intType = intTypeList.at(iter);

				IntegrationRange *intRange = m_integrationMap.value(intType);

				if(other.contains(intType, *intRange))
					continue;
				else
					return false;
			}

			return true;
		}


	bool
		HistoryItem::contains(IntegrationType intType,
				const IntegrationRange &intRange) const
		{
			IntegrationRange *localRange = m_integrationMap.value(intType, nullptr);

			if(localRange == nullptr)
				return false;

			if(localRange->start == intRange.start && localRange->end == intRange.end)
				return true;

			return false;
		}


	void
		HistoryItem::setDateTime()
		{
			m_dateTime = QDateTime::currentDateTimeUtc();
		}


	QDateTime
		HistoryItem::dateTime() const
		{
			return m_dateTime;
		}


	bool
		HistoryItem::hasIntegrationType(IntegrationType integrationType)
		{
			QList<IntegrationType> list = m_integrationMap.keys();

			if(list.contains(integrationType))
				return true;

			return false;
		}


	void
		HistoryItem::newIntegrationRange(IntegrationType intType, double start,
				double end)
		{
			IntegrationRange *intRange = new IntegrationRange;
			intRange->start = start;
			intRange->end = end;

			m_integrationMap.insert(intType, intRange);
		}


	IntegrationRange *
		HistoryItem::integrationRange(IntegrationType intType) const
		{
			IntegrationRange *intRange = m_integrationMap.value(intType, nullptr);
			return intRange;
		}


	bool
		HistoryItem::isValid() const
		{
			if(!m_dateTime.isValid())
				return false;

			if(m_integrationMap.isEmpty())
				return false;

			return true;
		}


	QString
		HistoryItem::asText(bool brief) const
		{
			QString text;

			// For each integration range in the map, craft a text element.

			QList<IntegrationRange *> list = m_integrationMap.values();

			//qDebug() << __FILE__ << __LINE__ << "There are " << list.size()
			//<< " integration ranges in the map";

			for(int iter = 0; iter < list.size(); ++iter)
			{
				IntegrationRange *intRange = list.at(iter);
				IntegrationType intType = m_integrationMap.key(intRange);

				//qDebug() << __FILE__ << __LINE__
				//<< "Currently iterated intType:" << intType;

				IntegrationTypeDesc *intTypeDesc = m_intTypeDescMap.value(intType);

				if(intTypeDesc == nullptr)
					qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);

				QString label;

				if(brief)
					label = intTypeDesc->brief + " ";
				else
					label = intTypeDesc->detailed + ":\n";

				if(brief)
				{
					text += label +
						QString("range: [%1-%2]")
						.arg(intRange->start, 0, 'f', 3)
						.arg(intRange->end, 0, 'f', 3);

					if(iter > 0 && iter < list.size() - 1)
						text += " | ";
					if(iter == list.size() - 1)
						text += "\n";
				}
				else
					text += label +
						QString("\tRange: [%1-%2]\n")
						.arg(intRange->start, 0, 'f', 3)
						.arg(intRange->end, 0, 'f', 3);
			}

			// Finally, only print the date time if we are verbose.
			if(!brief)
			{
				text += "\t" + m_dateTime.toString(Qt::ISODate);
				text += "\n";
			}


			// qDebug() << __FILE__ << __LINE__ << "text: " << text;

			return text;
		}


	History::History()
	{
	}


	History::History(const History &other)
	{
		for(int iter = 0; iter < other.m_itemList.size(); ++iter)
			m_itemList.append(new HistoryItem(*(other.m_itemList.at(iter))));
	}


	History &
		History::operator=(const History &other)
		{
			if(&other == this)
				return *this;

			for(int iter = 0; iter < other.m_itemList.size(); ++iter)
				m_itemList.append(new HistoryItem(*(other.m_itemList.at(iter))));

			return *this;
		}


	History::~History()
	{
		freeList();
	}


	void
		History::freeList()
		{
			while(m_itemList.size())
				delete m_itemList.takeFirst();
		}


	int
		History::containsHistoryItem(const HistoryItem &item)
		{
			for(int iter = 0; iter < m_itemList.size(); ++iter)
			{
				HistoryItem *curItem = m_itemList.at(iter);

				if(*curItem == item)
					return iter;
			}

			return -1;
		}


	HistoryItem *
		History::firstHistoryItem(IntegrationType intType,
				IntegrationRange **intRange)
		{
			// We want to return the first item that has integrationType.
			// We iterate in normal order because we want to return the first added
			// item.

			for(int iter = 0; iter < m_itemList.size(); ++iter)
			{
				HistoryItem *item = m_itemList.at(iter);

				if(item->hasIntegrationType(intType))
				{
					if(intRange != nullptr)
						*intRange = item->integrationRange(intType);

					return item;
				}
			}

			return nullptr;
		}


	HistoryItem *
		History::lastHistoryItem(IntegrationType intType, IntegrationRange **intRange)
		{
			// We want to return the last item that has integration type intType.
			// We iterate in reverse order because we want to return the last added
			// item.

			for(int iter = m_itemList.size() - 1; iter >= 0; --iter)
			{
				HistoryItem *item = m_itemList.at(iter);

				if(item->hasIntegrationType(intType))
				{
					if(intRange != nullptr)
						*intRange = item->integrationRange(intType);

					return item;
				}
			}

			return nullptr;
		}


	void
		History::appendHistory(const History &history, bool removeDuplicates)
		{

			// qDebug() << __FILE__ << __LINE__
			//<< "the history to be appended:" << history.asText();

			for(int iter = 0; iter < history.m_itemList.size(); ++iter)
			{
				HistoryItem *item = history.m_itemList.at(iter);

				if(containsHistoryItem(*item) >= 0 && removeDuplicates)
				{
					qDebug() << __FILE__ << __LINE__
						<< "Not appending history item as it would be duplicated.";
				}
				else
				{
					HistoryItem *newItem = new HistoryItem(*item);
					m_itemList.append(newItem);
				}
			}
		}


	int
		History::appendHistoryItem(HistoryItem *item)
		{
			if(item == nullptr)
				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);

			m_itemList.append(item);

			return m_itemList.size() - 1;
		}


	int
		History::duplicateHistoryItem(const HistoryItem &item)
		{
			HistoryItem *newItem = new HistoryItem(item);
			m_itemList.append(newItem);

			return m_itemList.size() - 1;
		}


	bool
		History::innermostRtRange(double *start, double *end) const
		{
			if(start == nullptr || end == nullptr)
				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);

			double tempStart = -1;
			double tempEnd = -1;

			bool itemFound = false;
			bool firstItem = true;

			QList<IntegrationType> intTypeList{IntegrationType::RT_TO_MZ,
				IntegrationType::RT_TO_DT,
				IntegrationType::RT_TO_TIC_INT,
				IntegrationType::ANY_TO_RT};

			// Iterate in the list of HistoryItem instances and for each check if there
			// is a (IntegrationType, IntegrationRange) pair in the QMap that matches
			// one of the RT_XXXX integration types. Thas is we search for integrations
			// that are defined *starting* from an RT range. If so store the start and
			// end values in temporary variables and make sure we finally get the
			// greatest start value and the smallest end value (we are seeking innermost
			// range values.

			for(int iter = 0; iter < m_itemList.size(); ++iter)
			{
				HistoryItem *item = m_itemList.at(iter);

				// Each history item my have one of the IntegrationType values related
				// to RT integration. Let's test them all in sequence.

				for(int iter = 0; iter < intTypeList.size(); ++iter)
				{
					IntegrationType intType = intTypeList.at(iter);

					IntegrationRange *intRange = item->integrationRange(intType);

					if(intRange != nullptr)
					{
						if(firstItem)
						{
							tempStart = intRange->start;
							tempEnd = intRange->end;

							firstItem = false;
						}
						else
						{
							// We seek the *innermost* range, so be aware of the
							// relational < >
							// operators.
							if(intRange->start > tempStart)
								tempStart = intRange->start;
							if(intRange->end < tempEnd)
								tempEnd = intRange->end;
						}

						// Let the caller know that we found at least one item and thus
						// that the start and end params now have the encountered
						// values.

						itemFound = true;
					}
					else
						// The returned intRange is nullptr.
						continue;
				}
			}
			// We have finally iterated in all the HistoryItem elements and we can
			// return the range values.
			*start = tempStart;
			*end = tempEnd;

			return itemFound;
		}


	bool
		History::innermostMzRange(double *start, double *end) const
		{
			if(start == nullptr || end == nullptr)
				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);

			double tempStart = -1;
			double tempEnd = -1;

			bool itemFound = false;
			bool firstItem = true;

			QList<IntegrationType> intTypeList{IntegrationType::MZ_TO_RT,
				IntegrationType::MZ_TO_DT,
				IntegrationType::DATA_TO_DT_MZ_MZ_RANGE,
				IntegrationType::MZ_TO_TIC_INT,
				IntegrationType::ANY_TO_MZ};

			for(int iter = 0; iter < m_itemList.size(); ++iter)
			{
				HistoryItem *item = m_itemList.at(iter);

				for(int iter = 0; iter < intTypeList.size(); ++iter)
				{
					IntegrationType intType = intTypeList.at(iter);

					IntegrationRange *intRange = item->integrationRange(intType);

					if(intRange != nullptr)
					{
						if(firstItem)
						{
							tempStart = intRange->start;
							tempEnd = intRange->end;

							firstItem = false;
						}
						else
						{
							// We seek the *innermost* range, so be aware of the
							// relational < >
							// operators.
							if(intRange->start > tempStart)
								tempStart = intRange->start;
							if(intRange->end < tempEnd)
								tempEnd = intRange->end;
						}

						// Let the caller know that we found at least one item and
						// thus
						// that
						// the start and end params now have the encountered values.
						itemFound = true;
					}
					else
						// The returned intRange is nullptr.
						continue;
				}
			}

			// We have finally iterated in all the HistoryItem elements and we can
			// return the range values.
			*start = tempStart;
			*end = tempEnd;

			return itemFound;
		}


	bool
		History::innermostDtRange(double *start, double *end) const
		{
			if(start == nullptr || end == nullptr)
				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);

			double tempStart = -1;
			double tempEnd = -1;

			bool itemFound = false;
			bool firstItem = true;

			QList<IntegrationType> intTypeList{IntegrationType::DT_TO_RT,
				IntegrationType::DT_TO_MZ,
				IntegrationType::DATA_TO_DT_MZ_DT_RANGE,
				IntegrationType::DT_TO_TIC_INT,
				IntegrationType::ANY_TO_DT};

			for(int iter = 0; iter < m_itemList.size(); ++iter)
			{
				HistoryItem *item = m_itemList.at(iter);

				//qDebug() << __FILE__ << __LINE__ << "Iterating in new history item.";

				for(int iter = 0; iter < intTypeList.size(); ++iter)
				{

					// Get one of the possible integration types related to DT.

					IntegrationType intType = intTypeList.at(iter);

					// Ask if the current HistoryItem has an integration range of that
					// type. It is perfectly possible that the returned pointer is
					// nullptr if the integration type asked for is not found.

					IntegrationRange *intRange = item->integrationRange(intType);

					if(intRange != nullptr)
					{
						//qDebug() << __FILE__ << __LINE__
						//<< "integration range for integration type ("
						//<< intType
						//<< ") = " << item->m_intTypeDescMap.value(intType)->brief
						//<< "is:"
						//<< "intRange start:" << intRange->start
						//<< "intRange end:" << intRange->end;

						if(firstItem)
						{
							tempStart = intRange->start;
							tempEnd = intRange->end;

							firstItem = false;
						}
						else
						{

							// We seek the *innermost* range, so be aware of the
							// relational < > operators.

							if(intRange->start > tempStart)
								tempStart = intRange->start;
							if(intRange->end < tempEnd)
								tempEnd = intRange->end;
						}

						// Let the caller know that we found at least one item and thus
						// that the start and end params now have the encountered
						// values.

						itemFound = true;
					}
					else

						// The returned intRange is nullptr, which means that the current
						// HistoryItem has no integration range by the queried
						// IntegrationType.

						continue;
				}
			}

			// We have finally iterated in all the HistoryItem elements and we can
			// return the range values.
			*start = tempStart;
			*end = tempEnd;

			return itemFound;
		}


	bool
		History::innermostRange(IntegrationType integrationType, double *start,
				double *end) const
		{
			if(integrationType & IntegrationType::RT_TO_ANY)
				return innermostRtRange(start, end);
			else if(integrationType & IntegrationType::MZ_TO_ANY)
				return innermostMzRange(start, end);
			else if(integrationType & IntegrationType::DT_TO_ANY)
				return innermostDtRange(start, end);

			return false;
		}


	QString
		History::innermostRangesAsText() const
		{
			QString text = "History - innermost ranges:\n";

			double startValue;
			double endValue;

			if(innermostRtRange(&startValue, &endValue))
				text += QString("RT range: [%1-%2]\n")
					.arg(startValue, 0, 'f', 3)
					.arg(endValue, 0, 'f', 3);
			if(innermostMzRange(&startValue, &endValue))
				text += QString("MZ range: [%1-%2]\n")
					.arg(startValue, 0, 'f', 5)
					.arg(endValue, 0, 'f', 5);
			if(innermostDtRange(&startValue, &endValue))
				text += QString("DT range: [%1-%2]\n")
					.arg(startValue, 0, 'f', 10)
					.arg(endValue, 0, 'f', 10);

			return text;
		}


	bool
		History::isValid() const
		{
			for(int iter = 0; iter < m_itemList.size(); ++iter)
			{
				if(!m_itemList.at(iter)->isValid())
					return false;
			}

			return true;
		}


	QString
		History::asText(const QString &header, const QString &footer,
				bool brief) const
		{
			QString text = header;

			for(int iter = 0; iter < m_itemList.size(); ++iter)
				text += m_itemList.at(iter)->asText(brief);

			text += footer;

			return text;
		}


} // namespace msXpertSuite
