/*   msXpertSuite - mass spectrometry software suite
 *   -----------------------------------------------
 *   Copyright(C) 2016 Filippo Rusconi
 *
 *   http://www.{site}.org
 *
 *   This file is part of the msXpertSuite project.
 *
 *   The msXpertSuite project is the successor of the massXpert project. This
 *   project now includes various independent modules:
 *   - massXpert, the original mass spectrometry environment software (greatly
 *     refactored code);
 *   - mineXpert, a powerful TIC chromatogram/mass/drift spectrum viewer/miner;
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MASS_SPEC_DATA_FILE_LOADER_HPP
#define MASS_SPEC_DATA_FILE_LOADER_HPP


/////////////////////// Qt includes
#include <QObject>
#include <QMainWindow>
#include <QStringList>
#include <QHash>
#include <QMap>


/////////////////////// Local includes
#include "globals.hpp"
#include "MassSpecDataSet.hpp"
#include "History.hpp"

#include <limits>

namespace msXpertSuite
{

  class MassSpecDataFileLoader : public QObject
  {

		Q_OBJECT

    public:
    QString     m_fileName       = "NOT_SET";
    QString     m_fieldSeparator = "NOT_SET";
    QStringList m_errorMessageList;
    QString     m_spectrumTitle    = "NOT_SET";
    bool        m_feedbackToStdout = false;

    uint m_spectrumCount = std::numeric_limits<uint>::max();

    // This switch is useful for the owner of an instance of this class to
    // stop the current operation.
    bool m_isOperationCancelled = false;

    MassSpecDataFileLoader(const QString &fileName = QString());
    virtual ~MassSpecDataFileLoader();

    void appendErrorMessage(QString errorMsg);
    void setFileName(const QString &fileName);
    virtual QString formatDescription() const = 0;
    virtual int     readSpectrumCount()       = 0;

    void cancelOperation();

    signals:
    void dataLoadMessageSignal(QString msg, int logType);
    void dataLoadProgressSignal(int value);
		
    void setupProgressFeedbackSignal(const QString &msg, int startValue,
                                     int endValue, int currentValue = -1);
    void updateProgressFeedbackSignal(const QString &msg, int currentValue);
  };
}

#endif // MASS_SPEC_DATA_FILE_LOADER_HPP
