/*   msXpertSuite - mass spectrometry software suite
 *   -----------------------------------------------
 *   Copyright(C) 2016 Filippo Rusconi
 *
 *   http://www.{site}.org
 *
 *   This file is part of the msXpertSuite project.
 *
 *   The msXpertSuite project is the successor of the massXpert project. This
 *   project now includes various independent modules:
 *   - massXpert, the original mass spectrometry environment software (greatly
 *     refactored code);
 *   - mineXpert, a powerful TIC chromatogram/mass/drift spectrum viewer/miner;
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef MASS_SPEC_DATA_FILE_LOADER_SQLITE3_HPP
#define MASS_SPEC_DATA_FILE_LOADER_SQLITE3_HPP

/////////////////////// Qt includes
#include <QtSql>
#include <QSqlDatabase>
#include <QStringList>

/////////////////////// Local includes
#include "globals.hpp"
#include "MassSpecDataSet.hpp"
#include "MassSpectrum.hpp"
#include "MassSpectrumList.hpp"
#include "History.hpp"
#include "MassSpecDataFileLoader.hpp"
#include "MassSpecSqlite3Handler.hpp"


namespace msXpertSuite
{

	class MassSpecDataFileLoaderSqlite3 : public MassSpecDataFileLoader
	{
		Q_OBJECT

		public:

			// We need this instance for all the SQLite3 database access (read)
			// work.

			MassSpecSqlite3Handler m_sqlite3Handler;
			MassSpecFileMetaData m_msFileMetaData;

			MassSpecFileMetaData massSpecFileMetaData();

			// How many spectra do we parse before providing a feedback message on the
			// progression of the parsing of the file?
			int m_parseProgressionNumber = 100;

			MassSpecDataFileLoaderSqlite3(const QString &fileName = QString());

			~MassSpecDataFileLoaderSqlite3();

			QString formatDescription() const override;

			static bool canLoadData(QString fileName);

			int readSpectrumCount() override;

			QList<double> *driftTimeList();
			void mzRange(double &mzStart, double &mzEnd);
			QList<double> rtList();


			int recordCountByScanStartTime(double start = 0, double end = -1);
			QList<double> scanStartTimeUniqueValues(double start = 0, double end = -1);

			bool isDriftExperiment();
			bool hasPrecursorIons();

			QString 
				craftSqlSelectStatementForStreamedIntegration(const History &history,
						bool withRecordCountStatement, bool integrateToDt);

			int streamedIntegration(const History &  history,
					IntegrationType  integrationType,
					QVector<double> *keyVector,
					QVector<double> *valVector);

			int streamedRtIntegration(const History &  history,
					QVector<double> *keyVector,
					QVector<double> *valVector);

			int streamedTicIntensity(const History &  history,
					double *intensity);

			int streamedMzIntegration(const History &  history,
					QVector<double> *keyVector,
					QVector<double> *valVector);

			int streamedDtIntegration(const History &  history,
					QVector<double> *keyVector,
					QVector<double> *valVector);

			int loadSpectra(int firstId, int lastId, MassSpecDataSet *massSpecDataSet, int &dataKind);

			int loadSpectra(const QString &  sqlQueryString,
					MassSpecDataSet *massSpecDataSet, int &dataKind);

			int loadSpectraByRt(double            retentionTime /*scanStartTime*/,
					MassSpectrumList *massSpectra);

			int loadData(MassSpecDataSet *massSpecDataSet);

			void chromatogram(const QString &title, QList<double> &timeList,
					QList<double> &iList);

	};
}
// namespace msXpertSuite

#endif /* MASS_SPEC_DATA_FILE_LOADER_SQLITE3_HPP */
