/*   msXpertSuite - mass spectrometry software suite
 *   -----------------------------------------------
 *   Copyright(C) 2016 Filippo Rusconi
 *
 *   http://www.{site}.org
 *
 *   This file is part of the msXpertSuite project.
 *
 *   The msXpertSuite project is the successor of the massXpert project. This
 *   project now includes various independent modules:
 *   - massXpert, the original mass spectrometry environment software (greatly
 *     refactored code);
 *   - mineXpert, a powerful TIC chromatogram/mass/drift spectrum viewer/miner;
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef MASS_SPEC_DATA_FILE_LOADER_XY_HPP
#define MASS_SPEC_DATA_FILE_LOADER_XY_HPP


////////////////////////////// Qt includes
#include <QStringList>
#include <QRegularExpression>

////////////////////////////// local includes
#include "MassSpecDataFileLoader.hpp"
#include "MassSpectrum.hpp"
#include "MassSpecDataSet.hpp"


namespace msXpertSuite
{

  class MassSpecDataFileLoaderXy : public MassSpecDataFileLoader
  {
    public:

			static QRegularExpression m_endOfLineRegExp;
			static QRegularExpression m_massDataRegExp;

    MassSpecDataFileLoaderXy(const QString &filename = QString());
    ~MassSpecDataFileLoaderXy();

    QString formatDescription() const override;
    int     readSpectrumCount() override;


    // Data loading
    static bool canLoadData(QString fileName);
    static bool canLoadDataFromString(const QString &msDataString);

		MassPeak *massPeakFromString(const QString &line);
    MassSpectrum *loadData();
    int loadDataFromString(const QString &msDataString, MassSpecDataSet *massSpecDataSet);
    int loadData(MassSpecDataSet *massSpecDataSet);
    int loadData(QVector<double> *keyVector,
                       QVector<double> *valVector);
    int loadData(QMap<double, double> *keyValMap);
  };


} // namespace msXpertSuite

#endif /* MASS_SPEC_DATA_FILE_LOADER_XY_HPP */
