/*   msXpertSuite - mass spectrometry software suite
 *   -----------------------------------------------
 *   Copyright(C) 2016 Filippo Rusconi
 *
 *   http://www.{site}.org
 *
 *   This file is part of the msXpertSuite project.
 *
 *   The msXpertSuite project is the successor of the massXpert project. This
 *   project now includes various independent modules:
 *   - massXpert, the original mass spectrometry environment software (greatly
 *     refactored code);
 *   - mineXpert, a powerful TIC chromatogram/mass/drift spectrum viewer/miner;
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


/////////////////////// Qt includes
#include <QDebug>
#include <QStringList>


/////////////////////// Local includes
#include "MassSpecFileMetaData.hpp"


namespace msXpertSuite
{

  MassSpecFileMetaData::MassSpecFileMetaData()
  {
  }

  MassSpecFileMetaData &
  MassSpecFileMetaData::operator=(const MassSpecFileMetaData &other)
  {
    if(&other == this)
      return *this;

    m_stringHash.clear();
    QHashIterator<QString, QString> stringHashIter(other.m_stringHash);
    while(stringHashIter.hasNext())
      {
        stringHashIter.next();
        setStringItem(stringHashIter.key(), stringHashIter.value());
      }

    m_intHash.clear();
    QHashIterator<QString, int> intHashIter(other.m_intHash);
    while(intHashIter.hasNext())
      {
        intHashIter.next();
        setIntItem(intHashIter.key(), intHashIter.value());
      }

    m_doubleHash.clear();
    QHashIterator<QString, double> doubleHashIter(other.m_doubleHash);
    while(doubleHashIter.hasNext())
      {
        doubleHashIter.next();
        setDoubleItem(doubleHashIter.key(), doubleHashIter.value());
      }

    m_boolHash.clear();
    QHashIterator<QString, bool> boolHashIter(other.m_boolHash);
    while(boolHashIter.hasNext())
      {
        boolHashIter.next();
        setBoolItem(boolHashIter.key(), boolHashIter.value());
      }

    return *this;
  }

  MassSpecFileMetaData::~MassSpecFileMetaData()
  {
  }


  bool
  MassSpecFileMetaData::stringItem(QString key, QString &value) const
  {
    if(key.isEmpty())
      return false;

    if(!m_stringHash.contains(key))
      return false;

    value = m_stringHash.value(key);

    return true;
  }


  int
  MassSpecFileMetaData::allStringItems(QStringList &keys,
                                       QStringList &values) const
  {
    keys.clear();
    values.clear();

    keys   = m_stringHash.uniqueKeys();
    values = m_stringHash.values();

    if(keys.size() != values.size())
      {
        qDebug() << __FILE__ << __LINE__ << "Error with the sizes of keys list "
                                            "and values list in hash. "
                                            "Exiting\n";

        exit(1);
      }

    return keys.size();
  }


  void
  MassSpecFileMetaData::setStringItem(QString key, QString value)
  {
    if(key.isEmpty())
      return;

    m_stringHash.insert(key, value);
  }


  bool
  MassSpecFileMetaData::intItem(QString key, int &value) const
  {
    if(key.isEmpty())
      return false;

    if(!m_intHash.contains(key))
      return false;

    value = m_intHash.value(key);

    return true;
  }


  int
  MassSpecFileMetaData::allIntItems(QStringList &keys, QList<int> &values) const
  {
    keys.clear();
    values.clear();

    keys   = m_intHash.uniqueKeys();
    values = m_intHash.values();

    if(keys.size() != values.size())
      {
        qDebug() << __FILE__ << __LINE__ << "Error with the sizes of keys list "
                                            "and values list in hash. "
                                            "Exiting\n";

        exit(1);
      }

    return keys.size();
  }


  void
  MassSpecFileMetaData::setIntItem(QString key, int value)
  {
    if(key.isEmpty())
      return;

    m_intHash.insert(key, value);
  }


  bool
  MassSpecFileMetaData::doubleItem(QString key, double &value) const
  {
    if(key.isEmpty())
      return false;

    if(!m_doubleHash.contains(key))
      return false;

    value = m_doubleHash.value(key);

    return true;
  }


  int
  MassSpecFileMetaData::allDoubleItems(QStringList &  keys,
                                       QList<double> &values) const
  {
    keys.clear();
    values.clear();

    keys   = m_doubleHash.uniqueKeys();
    values = m_doubleHash.values();

    if(keys.size() != values.size())
      {
        qDebug() << __FILE__ << __LINE__ << "Error with the sizes of keys list "
                                            "and values list in hash. "
                                            "Exiting\n";

        exit(1);
      }

    return keys.size();
  }


  void
  MassSpecFileMetaData::setDoubleItem(QString key, double value)
  {
    if(key.isEmpty())
      return;

    m_doubleHash.insert(key, value);
  }


  bool
  MassSpecFileMetaData::boolItem(QString key, bool &value) const
  {
    if(key.isEmpty())
      return false;

    if(!m_boolHash.contains(key))
      return false;

    value = m_boolHash.value(key);

    return true;
  }


  int
  MassSpecFileMetaData::allBoolItems(QStringList &keys,
                                     QList<bool> &values) const
  {
    keys.clear();
    values.clear();

    keys   = m_boolHash.uniqueKeys();
    values = m_boolHash.values();

    if(keys.size() != values.size())
      {
        qDebug() << __FILE__ << __LINE__ << "Error with the sizes of keys list "
                                            "and values list in hash. "
                                            "Exiting\n";

        exit(1);
      }

    return keys.size();
  }


  void
  MassSpecFileMetaData::setBoolItem(QString key, bool value)
  {
    if(key.isEmpty())
      return;

    m_boolHash.insert(key, value);
  }


  QString *
  MassSpecFileMetaData::asText(MassSpecFileMetaDataType type) const
  {
    QString *      p_text = new QString;
    QList<QString> keyList;
    QString        key;

    if(type & MassSpecFileMetaDataType::MASS_SPEC_META_DATA_TYPE_STRING)
      {
        keyList = m_stringHash.keys();

        for(int iter = 0; iter < keyList.size(); ++iter)
          {
            key           = keyList.at(iter);
            QString value = m_stringHash.value(key);

            // qDebug() << __FILE__ << __LINE__
            // << "key:value" << key << ":" << value;

            *p_text += QString("%1 = %2\n").arg(key).arg(value);
          }
      }

    if(type & MassSpecFileMetaDataType::MASS_SPEC_META_DATA_TYPE_INT)
      {
        keyList = m_intHash.keys();

        for(int iter = 0; iter < keyList.size(); ++iter)
          {
            key       = keyList.at(iter);
            int value = m_intHash.value(key);

            // qDebug() << __FILE__ << __LINE__
            // << "key:value" << key << ":" << value;

            *p_text += QString("%1 = %2\n").arg(key).arg(value);
          }
      }
    if(type & MassSpecFileMetaDataType::MASS_SPEC_META_DATA_TYPE_DOUBLE)
      {
        keyList = m_doubleHash.keys();

        for(int iter = 0; iter < keyList.size(); ++iter)
          {
            key       = keyList.at(iter);
            int value = m_doubleHash.value(key);

            // qDebug() << __FILE__ << __LINE__
            // << "key:value" << key << ":" << value;

            *p_text += QString("%1 = %2\n").arg(key).arg(value);
          }
      }


    if(type & MassSpecFileMetaDataType::MASS_SPEC_META_DATA_TYPE_BOOL)
      {
        keyList = m_boolHash.keys();

        for(int iter = 0; iter < keyList.size(); ++iter)
          {
            key        = keyList.at(iter);
            bool value = m_boolHash.value(key);

            // qDebug() << __FILE__ << __LINE__
            // << "key:value" << key << ":" << value;

            *p_text += QString("%1 = %2\n").arg(key).arg(value);
          }
      }

    // qDebug() << __FILE__ << __LINE__
    // << "*p_text:" << *p_text;

    return p_text;
  }

} // namespace msXpertSuite
