/*   msXpertSuite - mass spectrometry software suite
 *   -----------------------------------------------
 *   Copyright(C) 2016 Filippo Rusconi
 *
 *   http://www.{site}.org
 *
 *   This file is part of the msXpertSuite project.
 *
 *   The msXpertSuite project is the successor of the massXpert project. This
 *   project now includes various independent modules:
 *   - massXpert, the original mass spectrometry environment software (greatly
 *     refactored code);
 *   - mineXpert, a powerful TIC chromatogram/mass/drift spectrum viewer/miner;
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef MASS_SPECTRUM_LIST_HPP
#define MASS_SPECTRUM_LIST_HPP

/////////////////////// Qt includes
#include <QList>

/////////////////////// Local includes
#include "MassSpectrum.hpp"

namespace msXpertSuite
{

  class MassSpectrumList : public QList<MassSpectrum *>
  {
    public:
    QString m_title;

    MassSpectrumList(const QString &title = QString());
    MassSpectrumList(const QList<MassSpectrum *> &massSpectra);

    ~MassSpectrumList();

    MassSpectrumList &operator=(const MassSpectrumList &massSpectra);
    MassSpectrumList &operator=(const QList<MassSpectrum *> &massSpectra);
    void freeSpectra();
    void deleteMassPeaks();

		void characters(double *minMz, double *maxMz, int *greatestSize,
			 QList<double> *stepList,	double *greatestStepOfAll) const;
		bool sameLengthSpectra() const;

    double tic();
    double tic(double mzStart, double mzEnd);
    static double tic(const MassSpectrumList &massSpectra);

    static int combine(const MassSpectrumList &massSpectra,
                       MassSpectrum &          combinedMassSpectrum);
    MassSpectrum *combine();
    MassSpectrum *combine(double mzStart, double mzEnd);

    static int keepCommonSpectra(MassSpectrumList &      where,
                                 const MassSpectrumList &list);
  };


} // namespace msXpertSuite

#endif /* MASS_SPECTRUM_LIST_HPP */
