/*   msXpertSuite - mass spectrometry software suite
 *   -----------------------------------------------
 *   Copyright(C) 2016 Filippo Rusconi
 *
 *   http://www.{site}.org
 *
 *   This file is part of the msXpertSuite project.
 *
 *   The msXpertSuite project is the successor of the massXpert project. This
 *   project now includes various independent modules:
 *   - massXpert, the original mass spectrometry environment software (greatly
 *     refactored code);
 *   - mineXpert, a powerful TIC chromatogram/mass/drift spectrum viewer/miner;
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef SQL_MASS_DATA_SLICER_HPP
#define SQL_MASS_DATA_SLICER_HPP


/////////////////////// Qt includes
#include <QString>


/////////////////////// Local includes
#include "MassSpecSqlite3Handler.hpp"
#include "History.hpp"


namespace msXpertSuite
{

  enum SliceType
  {
    SLICE_NOT_SET,
    SLICE_ZOOMED_REGION,
    SLICE_BY_COUNT,
    SLICE_BY_SIZE,
  };


  struct SliceSpecif
  {
    QString fileName;
    double  start;
    double  end;
    int     number;
  };


  class SqlMassDataSlicer : public QObject
  {
    Q_OBJECT

    public:
    SqlMassDataSlicer(const SqlMassDataSlicer &other);
    SqlMassDataSlicer(MassSpecSqlite3Handler *sqlHandler, History history,
                      double rangeStart, double rangeEnd);
    ~SqlMassDataSlicer();

    // We do not own this
    MassSpecSqlite3Handler *mp_sqlHandler = nullptr;
    History                 m_history;

    // The file name of the original data.
    QString m_srcFileName;

    // The destination directory.
    QString m_destDirectory;

    // The string that describes how the destination files should be named
    // depending on how the slicing should be carried over.
    QString m_formatString;

    // List of specifications that hold all the details of how the slicing
    // should occur.
    QList<SliceSpecif *> m_sliceSpecifList;

    int m_sliceType = SliceType::SLICE_NOT_SET;

    double m_rangeStart;
    double m_rangeEnd;

    int    m_sliceCount;
    double m_sliceSize;

    SqlMassDataSlicer &operator=(const SqlMassDataSlicer &other);

    bool fillInSpecifs(QString *errors);
    QString craftSliceFileName(SliceSpecif *sliceSpecif);
    QString allFileNamesAsString();

    bool slice();

    QString asText() const;
    QString sliceSpecifAsText(SliceSpecif *specif) const;
    QString allSliceSpecifsAsText() const;

    void clearSliceSpecifications();
    void reset();

    signals:
    void updateSliceFileName(QString fileName);
  };

} // namespace msXpertSuite


#endif // SQL_MASS_DATA_SLICER_HPP
