/*   msXpertSuite - mass spectrometry software suite
 *   -----------------------------------------------
 *   Copyright(C) 2016 Filippo Rusconi
 *
 *   http://www.{site}.org
 *
 *   This file is part of the msXpertSuite project.
 *
 *   The msXpertSuite project is the successor of the massXpert project. This
 *   project now includes various independent modules:
 *   - massXpert, the original mass spectrometry environment software (greatly
 *     refactored code);
 *   - mineXpert, a powerful TIC chromatogram/mass/drift spectrum viewer/miner;
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef __SQL_TABLE_FIELD_SPECIFS__
#define __SQL_TABLE_FIELD_SPECIFS__

#include <QString>

namespace msXpertSuite
{

  enum FieldQtType
  {
    QT_TYPE_NOT_SET = 0,
    QSTRING,
    INT,
    DOUBLE,
    BINARY,
  };

  enum FieldSqlType
  {
    SQL_TYPE_NOT_SET,
    TEXT,
    INTEGER,
    FLOAT,
    BLOB,
  };


  class SqlTableFieldSpecifs
  {
    private:
    QString m_tableName;
    QString m_fieldName;

    FieldQtType  m_fieldQtType;
    FieldSqlType m_fieldSqlType;

    QString m_comment;
    QString m_commandLine;

    public:
    SqlTableFieldSpecifs(const QString &tableName   = QString(),
                         const QString &fieldName   = QString(),
                         FieldQtType                = QT_TYPE_NOT_SET,
                         FieldSqlType               = SQL_TYPE_NOT_SET,
                         const QString &comment     = QString(),
                         const QString &commandLine = QString());

    SqlTableFieldSpecifs(const QString &tableName);

    ~SqlTableFieldSpecifs();

    void setTableName(QString tableName);
    QString tableName() const;

    void setFieldName(QString fieldName);
    QString fieldName() const;

    void setFieldQtType(FieldQtType type);
    FieldQtType fieldQtType() const;

    void setFieldSqlType(FieldSqlType type);
    FieldSqlType fieldSqlType() const;

    void setComment(QString comment);
    QString comment() const;

    void setCommandLine(QString commandLine);
    QString commandLine() const;
  };

} // namespace msXpertSuite

#endif /* __SQL_TABLE_FIELD_SPECIFS__ */
