/*   msXpertSuite - mass spectrometry software suite
 *   -----------------------------------------------
 *   Copyright(C) 2016 Filippo Rusconi
 *
 *   http://www.{site}.org
 *
 *   This file is part of the msXpertSuite project.
 *
 *   The msXpertSuite project is the successor of the massXpert project. This
 *   project now includes various independent modules:
 *   - massXpert, the original mass spectrometry environment software (greatly
 *     refactored code);
 *   - mineXpert, a powerful TIC chromatogram/mass/drift spectrum viewer/miner;
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef ABSTRACT_MULTI_PLOT_WND_HPP
#define ABSTRACT_MULTI_PLOT_WND_HPP

///////////////////////////// Qt includes
#include <QtCore/QVariant>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QStatusBar>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>


///////////////////////////// Local includes
#include "AbstractPlotWidget.hpp"
#include "MassSpecDataSet.hpp"


namespace msXpertSuite
{

	class AbstractMultiPlotWnd : public QMainWindow
	{
		Q_OBJECT

		public:
			// Construction/Destruction
			AbstractMultiPlotWnd(QWidget *parent, const QString &name,
					const QString &desc);

			virtual ~AbstractMultiPlotWnd();

			void writeSettings();
			void readSettings();

			void closeEvent(QCloseEvent *event) override;

			QLabel *mp_statusBarLabel = nullptr;
			QProgressBar *mp_statusBarProgressBar = nullptr;
			QPushButton *mp_statusBarCancelPushButton = nullptr;

			bool m_isOperationCancelled = false;

			QString m_name = "NOT_SET";
			Q_PROPERTY(QString m_name WRITE setName READ name);

			QString m_desc = "NOT_SET";
			Q_PROPERTY(QString m_desc WRITE setDesc READ desc);

			QList<AbstractPlotWidget *> m_plotWidgetList;
			QWidget *mp_lastFocusedPlotWidget = nullptr;

			bool delistPlotWidget(AbstractPlotWidget *plotWidget);

			// We want to maintain a connection between each given plot widget in the
			// window and its replica in the multigraph plot widget.
			QMap<AbstractPlotWidget *, QCPGraph *> m_plotWidgetGraphMap;


			virtual AbstractPlotWidget *
				addPlotWidget(QVector<double> keyVector, QVector<double> valVector,
						const QString & desc = QString(),
						MassSpecDataSet *massSpecDataSet = nullptr) = 0;

			QCPGraph *multiGraphReplicaGraph(AbstractPlotWidget *plotWidget);


			bool setupWindow();

			void setupProgressFeedback(QString msg, int startValue, int endValue,
					int currentValue = -1);
			void updateProgressFeedback(QString msg, int currentValue = -1);
			void statusBarMessage(QString msg);
			void statusBarProgress(int value);


			QWidget *mp_centralWidget = nullptr;
			QGridLayout *mp_centralWidgetGridLayout = nullptr;
			QSplitter *mp_splitter = nullptr;
			QScrollArea *mp_scrollArea = nullptr;

			QWidget *mp_scrollAreaWidgetContents = nullptr;
			QGridLayout *mp_scrollAreaWidgetContentsGridLayout = nullptr;
			QVBoxLayout *mp_scrollAreaVBoxLayout = nullptr;

			QColor m_wishedColor = Qt::black;

			AbstractPlotWidget *plotWidgetAt(int index);
			int plotWidgetIndex(AbstractPlotWidget *plotWidget);

			virtual void clearPlots();

			QMenuBar *mp_menubar = nullptr;
			QStatusBar *mp_statusbar = nullptr;
			QToolBar *mp_toolbar = nullptr;

			QAction *mp_lockXRangeAct = nullptr;
			bool m_lockXRange = false;

			QAction *mp_lockYRangeAct = nullptr;
			bool m_lockYRange = false;

			void lockXRangeToggle(bool checked);
			void lockYRangeToggle(bool checked);

			void keyReleaseEvent(QKeyEvent *event) override;
			virtual void emitReplotSignal(QCPRange xAxisRange, QCPRange yAxisRange);
			virtual void emitRedrawPlotBackground(QWidget *focusedPlotWidget);

			virtual void toggleMultiGraphGraph(AbstractPlotWidget *widget);
			virtual void showMultiGraphGraph(AbstractPlotWidget *widget);
			virtual void hideMultiGraphGraph(AbstractPlotWidget *widget);

			void removeMultiGraphGraph(AbstractPlotWidget *widget);

			void cancelPushButtonClicked();

			public slots:
				void setName(QString name);
			QString name();

			void setDesc(QString desc);
			QString desc();

signals:
			void aboutToDestroyPlotWidget(AbstractPlotWidget *widget);

			void replotWithAxisRange(QCPRange xAxisRange, QCPRange yAxisRange,
					int whichAxis);

			void redrawPlotBackground(QWidget *focusedPlotWidget);
			void recyclePlottingColor(QColor color);
			void cancelOperationSignal();

			public slots:
				void updateStatusBar(QString msg);
	};


} // namespace msXpertSuite


#endif // ABSTRACT_MULTI_PLOT_WND_HPP
