/*   msXpertSuite - mass spectrometry software suite
 *   -----------------------------------------------
 *   Copyright(C) 2016 Filippo Rusconi
 *
 *   http://www.{site}.org
 *
 *   This file is part of the msXpertSuite project.
 *
 *   The msXpertSuite project is the successor of the massXpert project. This
 *   project now includes various independent modules:
 *   - massXpert, the original mass spectrometry environment software (greatly
 *     refactored code);
 *   - mineXpert, a powerful TIC chromatogram/mass/drift spectrum viewer/miner;
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef ABSTRACT_MAIN_TASK_WINDOW_HPP
#define ABSTRACT_MAIN_TASK_WINDOW_HPP

/////////////////////// Qt includes
#include <QMainWindow>


/////////////////////// Local includes
#include "UserSpec.hpp"
#include "ConfigSettings.hpp"


namespace msXpertSuite
{

  class MainWindow;


  class AbstractMainTaskWindow : public QMainWindow
  {
    protected:
    MainWindow *mp_parentWnd;

    QString m_wndTypeName;
    QString m_wndTitle;

    virtual void readSettings();
    virtual void writeSettings();

    virtual bool initialize() = 0;

    protected slots:
    virtual void closeEvent(QCloseEvent *event);

    public:
    AbstractMainTaskWindow(MainWindow *parent, const QString &wndTypeName,
                           const QString &wndTitle);

    ~AbstractMainTaskWindow();

    QString               configSettingsFilePath();
    const UserSpec &      userSpec();
    const ConfigSettings *configSettings() const;

    const MainWindow *mainWindow() const;
  };

} // namespace msXpertSuite

#endif // ABSTRACT_MAIN_TASK_WINDOW_HPP
