/*   msXpertSuite - mass spectrometry software suite
 *   -----------------------------------------------
 *   Copyright(C) 2016 Filippo Rusconi
 *
 *   http://www.{site}.org
 *
 *   This file is part of the msXpertSuite project.
 *
 *   The msXpertSuite project is the successor of the massXpert project. This
 *   project now includes various independent modules:
 *   - massXpert, the original mass spectrometry environment software (greatly
 *     refactored code);
 *   - mineXpert, a powerful TIC chromatogram/mass/drift spectrum viewer/miner;
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


/////////////////////// Local includes
#include "CleaveOligomer.hpp"


namespace msXpertSuite
{

  CleaveOligomer::CleaveOligomer(Polymer *polymer, const QString &name,
                                 const QString &description, bool modified,
                                 const Ponderable &ponderable, int startIndex,
                                 int endIndex, int partialCleavage,
                                 const CalcOptions &calcOptions)
    : Oligomer(polymer, name, description, modified, ponderable, startIndex,
               endIndex, calcOptions)
    , m_partialCleavage(partialCleavage)
  {
  }


  CleaveOligomer::CleaveOligomer(Polymer *polymer, const QString &name,
                                 const QString &description, bool modified,
                                 double mono, double avg, int startIndex,
                                 int endIndex, int partialCleavage,
                                 const CalcOptions &calcOptions)
    : Oligomer(polymer, name, description, modified, mono, avg, startIndex,
               endIndex, calcOptions)
    , m_partialCleavage(partialCleavage)
  {
  }


  CleaveOligomer::CleaveOligomer(const CleaveOligomer &oligomer)
    : Oligomer(oligomer), m_partialCleavage(oligomer.m_partialCleavage)
  {
  }


  void
  CleaveOligomer::setPartialCleavage(int value)
  {
    m_partialCleavage = value;
  }

  int
  CleaveOligomer::partialCleavage() const
  {
    return m_partialCleavage;
  }

} // namespace msXpertSuite
