/*   msXpertSuite - mass spectrometry software suite
 *   -----------------------------------------------
 *   Copyright(C) 2016 Filippo Rusconi
 *
 *   http://www.{site}.org
 *
 *   This file is part of the msXpertSuite project.
 *
 *   The msXpertSuite project is the successor of the massXpert project. This
 *   project now includes various independent modules:
 *   - massXpert, the original mass spectrometry environment software (greatly
 *     refactored code);
 *   - mineXpert, a powerful TIC chromatogram/mass/drift spectrum viewer/miner;
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef COLOR_MAP_WND_HPP
#define COLOR_MAP_WND_HPP

/////////////////////// Qt includes
#include <QObject>
#include <QString>
#include <QWidget>
#include <QColor>
#include <QHash>


/////////////////////// Local includes
#include "AbstractMultiPlotWnd.hpp"
#include "ColorMapPlotWidget.hpp"
#include "AnalysisPreferences.hpp"
#include "MassSpecDataSet.hpp"


namespace msXpertSuite
{

	class ColorMapWnd : public AbstractMultiPlotWnd
	{
		Q_OBJECT

		public:
			QFile *mp_analysisFile = nullptr;
			// The analysis preferences are set from the MainWindow when the user sets
			// them with the dedicated dialog window.
			AnalysisPreferences *mp_analysisPreferences = nullptr;
			ColorMapPlotWidget * mp_multiGraphPlotWidget = nullptr;
			ColorMapWnd(QWidget *parent);
			~ColorMapWnd();

			void initialize();
			void initializePlotRegion();

			Q_INVOKABLE void hide(){QMainWindow::hide();};
			Q_INVOKABLE void show(){QMainWindow::show();};

			AbstractPlotWidget *addPlotWidget(QVector<double> keyVector,
					QVector<double> valVector,
					const QString & desc,
					MassSpecDataSet *massSpecDataSet);

			AbstractPlotWidget *initialColorMap(MassSpecDataSet *massSpecDataSet,
					bool * ok);

			void ticIntensity(MassSpecDataSet *massSpecDataSet,
					double *intensity, IntegrationType integrationType, History history);

			void streamedTicIntensity(MassSpecDataSet *massSpecDataSet,
					double *intensity, IntegrationType integrationType, History history);

			bool focusNextPrevChild(bool next);
	};

} // namespace msXpertSuite

#endif /* COLOR_MAP_WND_HPP */
