/*   msXpertSuite - mass spectrometry software suite
 *   -----------------------------------------------
 *   Copyright(C) 2016 Filippo Rusconi
 *
 *   http://www.{site}.org
 *
 *   This file is part of the msXpertSuite project.
 *
 *   The msXpertSuite project is the successor of the massXpert project. This
 *   project now includes various independent modules:
 *   - massXpert, the original mass spectrometry environment software (greatly
 *     refactored code);
 *   - mineXpert, a powerful TIC chromatogram/mass/drift spectrum viewer/miner;
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#pragma once


/////////////////////// Qt includes
#include <QObject>
#include <QString>
#include <QWidget>


///////////////////////////// Local includes
#include "AbstractPlotWidget.hpp"
#include "History.hpp"


namespace msXpertSuite
{


	class TicChromPlotWidget : public AbstractPlotWidget
	{
		Q_OBJECT

		public:
			QString craftAnalysisStanza();
			QString craftAnalysisStanza(QCPGraph *graph, const QString &fileName);

			IntegrationType arbitraryIntegrationType();

			// Construction/Desctruction
			TicChromPlotWidget(QWidget *parent, QWidget *parentWnd, const QString &name,
					const QString &desc, const QString &fileName = QString(),
					MassSpecDataSet *massSpecDataSet = nullptr);

			~TicChromPlotWidget();

			void mouseReleaseHandledEvent(QMouseEvent *       event,
					AbstractPlotWidget *emitter);
			void keyReleaseEvent(QKeyEvent *event);

			Q_INVOKABLE void integrateToMz(double lower, double upper);
			Q_INVOKABLE void integrateToDt(double lower, double upper);
			Q_INVOKABLE double integrateToTicIntensity(double lower, double upper);

signals:
			void recordAnalysisStanza(QString stanza, const QColor &color = QColor());

			void newMassSpectrumByRt(MassSpecDataSet *massSpecDataSet,
					IntegrationType integrationType,
					History history, QColor color);

			void newDriftSpectrumByRt(MassSpecDataSet *massSpecDataSet,
					IntegrationType integrationType,
					History history, QColor color);
	};


} // namespace msXpertSuite

