/*   msXpertSuite - mass spectrometry software suite
 *   -----------------------------------------------
 *   Copyright(C) 2016 Filippo Rusconi
 *
 *   http://www.{site}.org
 *
 *   This file is part of the msXpertSuite project.
 *
 *   The msXpertSuite project is the successor of the massXpert project. This
 *   project now includes various independent modules:
 *   - massXpert, the original mass spectrometry environment software (greatly
 *     refactored code);
 *   - mineXpert, a powerful TIC chromatogram/mass/drift spectrum viewer/miner;
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef COMMAND_LINE_PARSER_HPP
#define COMMAND_LINE_PARSER_HPP


/////////////////////// Qt includes
#include <QObject>
#include <QList>
#include <QStringList>


/////////////////////// Local includes
#include "History.hpp"


namespace msXpertSuite
{

  class CommandLineRunner : public QObject
  {
    Q_OBJECT

    public:
    bool m_subtract = false;

    bool    m_isStreamed = false;
    QString m_integration;

    QStringList m_inFileNames;
    QString     m_outFileName;
    QString     m_scriptFileName;
    bool        m_isStdOut = false;

    double m_rtStart, m_rtEnd;
    double m_mzStart, m_mzEnd;
    double m_dtStart, m_dtEnd;

    History m_history;

    QString m_currentInputFileName;
    QString m_baseMessage;

    CommandLineRunner();
    virtual ~CommandLineRunner();

    bool allStartEndValuesUnset();

    bool createHistoryItems(QString *errors);

    QStringList existingInputFiles();
    bool allInputFilesExist(int *existingFiles, int *missingFiles);
    bool allInputFilesExist(QStringList *existingFiles,
                            QStringList *missingFiles);
    bool shouldRunGuiRightAway(QStringList *existingFiles, QString *errors);

    bool run(QStringList *existingFiles, QString *errors, bool *shouldRunGui);

    bool run_RT_ARBITRARY(QString *errors);
    bool runSum_RT_ARBITRARY(QString *errors);
    bool runSubtract_RT_ARBITRARY(QString *errors);

    bool run_MZ_ARBITRARY(QString *errors);
    bool runSum_MZ_ARBITRARY(QString *errors);
    bool runSubtract_MZ_ARBITRARY(QString *errors);

    bool run_DT_ARBITRARY(QString *errors);
    bool runSum_DT_ARBITRARY(QString *errors);
    bool runSubtract_DT_ARBITRARY(QString *errors);

    void setupProgressFeedback(QString msg, int startValue, int endValue,
                               int currentValue = -1);
    void updateProgressFeedback(QString msg, int currentValue);

    int writeOutput(const QMap<double, double> &keyValMap);

    QString asText();
  };


} // namespace msXpertSuite

#endif // COMMAND_LINE_PARSER_HPP
